function [rel_IAE, IAE, time, peak] = controller_executions_2(hp, dt, Y, T)

    %Ziegler Nichols
    [hc,Kp,Ti]=zn_pi(hp);
    hp=zpk(hp);
    [Yr,t]=step(hp*hc/(hc*hp+1));
    ZN_time = t(end);
    ZN_peak = max(Yr, [], 'all');
    ZN_rel_IAE = sum(abs(Yr-1))/sum(1*length(Yr));
    if ZN_rel_IAE > 1 || isnan(ZN_rel_IAE)
        ZN_rel_IAE = -1;
    end
    ZN_IAE = sum(length(Yr)) - sum(abs(Yr));

    %PID-tuner
    zeta=1;
    delta=2.3;
    [Kp,Ti,Td]=delta_prc_pid_tun1....
    (T,Y,dt,delta,zeta);hc=pidstd(Kp,Ti,Td);rho=delta;
    hp=zpk(hp);
    [Yr,t]=step(hp*hc/(hc*hp+1));
    PID_peak = max(Yr, [], 'all');
    PID_time = t(end);
    PID_rel_IAE = sum(abs(Yr-1))/sum(1*length(Yr));
    if PID_rel_IAE > 1 || isnan(PID_rel_IAE)
        PID_rel_IAE = -1;
    end
    PID_IAE = sum(length(Yr)) - sum(abs(Yr));
    
    %PI-tuner
    zeta=1;
    delta=2.3;
    [Kp,Ti]=delta_prc_pi_tun(T,Y,dt,delta,zeta);
    hc=pidstd(Kp,Ti);
    rho=delta;
    hp=zpk(hp);
    [Yr,t]=step(hp*hc/(hc*hp+1));
    PI_peak = max(Yr, [], 'all');
    PI_time = t(end);
    PI_rel_IAE = sum(abs(Yr-1))/sum(1*length(Yr));
    if PI_rel_IAE > 1 || isnan(PI_rel_IAE)
        PI_rel_IAE = -1;
    end
    PI_IAE = sum(length(Yr)) - sum(abs(Yr));
    
    %Plain Megatuner
    rho=2; kp_alt=1;
    hc=megatuner1plain(hp,rho,kp_alt);
    [Yr,t]=step(hp*hc/(hc*hp+1));
    Plain_peak = max(Yr, [], 'all');
    Plain_time = t(end);
    Plain_rel_IAE = sum(abs(Yr-1))/sum(1*length(Yr));
    if Plain_rel_IAE > 1 || isnan(Plain_rel_IAE)
        Plain_rel_IAE = -1;
    end
    Plain_IAE = sum(length(Yr)) - sum(abs(Yr));
    
    %Megatuner
    [hc,rho]=megatuner1(hp); % heuristic ...
    [Yr,t]=step(hp*hc/(hc*hp+1));
    Mega_peak = max(Yr, [], 'all'); 
    Mega_time = t(end);
    Mega_rel_IAE = sum(abs(Yr-1))/sum(1*length(Yr));
    if Mega_rel_IAE > 1 || isnan(Mega_rel_IAE)
        Mega_rel_IAE = -1;
    end
    Mega_IAE = sum(length(Yr)) - sum(abs(Yr));

    rel_IAE = [ZN_rel_IAE, PID_rel_IAE, PI_rel_IAE, Plain_rel_IAE, Mega_rel_IAE];
    IAE = [ZN_IAE, PID_IAE, PI_IAE, Plain_IAE, Mega_IAE];
    time = [ZN_time, PID_time, PI_time, Plain_time, Mega_time];
    peak = [ZN_peak, PID_peak, PI_peak, Plain_peak, Mega_peak];

end